/*
 * Copyright (c) 2007 by EverNote Corporation, All rights reserved.
 */

/*
 * This file contains the EDAM protocol interfaces for operations that may
 * only be used by trusted components of the service.  These interfaces
 * are not exposed outside the firewall for external clients.
 */

include "Errors.thrift"
include "Types.thrift"
include "NoteStore.thrift"
include "UserStore.thrift"

namespace java com.evernote.edam.internal
namespace csharp Evernote.EDAM.Internal
namespace py evernote.edam.internal
namespace cpp evernote.edam
namespace rb Evernote.EDAM.Internal
namespace perl EDAMInternal
namespace php EDAM.Internal

// ============================= Enumerations ==================================

/**
 * Enumeration for the types of clients represented in the activity logs
 */
enum ClientType {
  WEB = 1,
  MOBILE_WEB = 2,
  SYNC = 3,
  PUBLIC_WEB = 4,
  EXTERNAL_SERVICE = 5
}

/**
 * Enumeration for the types of actions that will be recorded in the
 * activity logs.
 */
enum ActivityAction {
  LOGIN = 1,
  FORGOT_PASSWORD = 2,
  FAILED_LOGIN = 3,
  RESET_PASSWORD = 4,
  CHANGE_EMAIL = 5
}

/**
 * AdFilters can either be used to require a match with a client property or
 * to reject ads for clients who specify a property
 */
enum AdFilterType {
  REJECT = 0,
  REQUIRED = 1
}

/**
 * Enumeration of the different independent access rights that may be associated
 * with an authentication token.  This determines which operations may be
 * performed by the bearer of that token.
 */
enum AuthenticationPermission {
  NOTESTORE_CREATE = 1,
  NOTESTORE_READ = 2,
  NOTESTORE_UPDATE = 4,
  NOTESTORE_EXPUNGE_NONNOTE = 8,  // Expunge anything except for Notes
  USERSTORE_READ = 16,
  USERSTORE_REFRESH = 32,
  NOTESTORE_EXPUNGE_NOTE = 64  // Expunge Notes
}

/**
 * Enumeration of the different types of user tasks that can be queued
 */
enum UserTaskType {
  RECOGNIZE_PDFS = 1,
  FLUSH_USER_DATA_CACHE = 3
}

/**
 * Enumeration of Sponsor Roles
 */
enum SponsorRole {
  MEMBER = 1,
  ADMIN = 2,
  OWNER = 3
}

/**
 * Enumeration of Sponsor Billing Types
 */
enum BillingType {
  SUSPENDED = 1,
  CREDIT_CARD = 2,
  INVOICE = 3,
  CANCELED = 4
}

/**
 * Enumeration of Sponsor User Status
 */
enum SponsorUserStatus {
  PENDING = 1,
  ACTIVE = 2,
  DEACTIVATED = 3,
  BLOCKED = 4,
  REMOVED = 5,
  DEACTIVATION_PENDING = 6
}

/**
 * This is a constant that can be used with createAuthenticationToken to
 * indicate that the token should be created with full rights to all
 * aspects of the user's account.
 */
const list<AuthenticationPermission> AUTH_PERMISSIONS_ALL = [
  AuthenticationPermission.NOTESTORE_CREATE,
  AuthenticationPermission.NOTESTORE_READ,
  AuthenticationPermission.NOTESTORE_UPDATE,
  AuthenticationPermission.NOTESTORE_EXPUNGE_NONNOTE,
  AuthenticationPermission.USERSTORE_READ,
  AuthenticationPermission.USERSTORE_REFRESH,
  AuthenticationPermission.NOTESTORE_EXPUNGE_NOTE
];

/**
 * Internal User Attribute Enums
 * These values are used in the DB.
 * You must *NEVER* change their integer values.
 * You *MAY* only add new values.
 * You *MAY* rename values.
 * You *MUST* never remove a value. Add a comment about it being deprecated.
 */
enum InternalUserAttributeType {
  MARKETING_EMAIL_PREFERENCES = 1, /* Deprecated */
  GROUP_BUSINESS_ADDRESS = 2,
  MARKETING_EMAIL_PREFERENCE_NEWS = 10,
  MARKETING_EMAIL_PREFERENCE_MONTHLY = 11,
  MARKETING_EMAIL_PREFERENCE_BUSINESS = 12,
  MARKETING_EMAIL_PREFERENCE_EDUCATION = 13,
  MARKETING_EMAIL_PREFERENCE_SURVEYS = 14,
  MARKETING_EMAIL_PREFERENCE_SKITCH = 20,
  MARKETING_EMAIL_PREFERENCE_CLEARLY = 21,
  MARKETING_EMAIL_PREFERENCE_FOOD = 22,
  MARKETING_EMAIL_PREFERENCE_HELLO = 23,
  MARKETING_EMAIL_PREFERENCE_PEEK = 24,
  AMAZON_PURCHASE_TOKEN=30
}

const i32    EDAM_INTERNAL_USER_ATTRIBUTE_VALUE_LEN_MIN = 0;
const i32    EDAM_INTERNAL_USER_ATTRIBUTE_VALUE_LEN_MAX = 4096;


/**
 * The minimum length of a related notes term when it is a single word
 * or prefix (and not counting the trailing asterisk for a prefix query).
 */
const i32    EDAM_RELATEDNOTES_TERM_LEN_MIN = 1;

/**
 * The maximum length of a related notes term when it is a single word
 * or prefix (and not counting the trailing asterisk for a prefix query).
 */
const i32    EDAM_RELATEDNOTES_TERM_LEN_MAX = 64;
                                           
/**
 * The pattern that a single word term must match, less a trailing asterisk
 * for a prefix query.
 */
const string EDAM_RELATEDNOTES_TERM_REGEX = "^[^\\p{Cc}\\p{Z}]{1,64}"; 

/**
 * The minimum length of the text for a related notes term for a phrase query.
 */
const i32    EDAM_RELATEDNOTES_PHRASE_LEN_MIN = 1;

/**
 * The maximum length of the text for a related notes term for a phrase query.
 */
const i32    EDAM_RELATEDNOTES_PHRASE_LEN_MAX = 256;
                                           
/**
 * The pattern that the text for a related notes term for a phrase query
 * must match.
 */
const string EDAM_RELATEDNOTES_PHRASE_REGEX =
  "^[^\\p{Cc}\\p{Z}]([^\\p{Cc}\\p{Zl}\\p{Zp}]{0,254}[^\\p{Cc}\\p{Z}])?$";                                                                       

/**
 * The largest pad, in miles, for the latitude and longitude for related
 * notes search.  The pad is added to each side of the "center" to perform
 * a search.
 */
const double EDAM_RELATEDNOTES_GEOGRAPHIC_MAX_PAD_MILES = 25.0;

/**
 * The maximum number of query terms that the server will allow from a
 * related notes query that comes from the example note or example text.
 */
const i32 EDAM_RELATEDNOTES_MAX_TERMS_FROM_EXAMPLE = 64;

/**
 * This is the smallest permitted slop value for searching for related notes
 * with phrases.
 */
const i32 EDAM_RELATEDNOTES_PHRASE_SLOP_MIN = 0;

/**
 * This is the largest permitted slop value for searching for related notes
 * with phrases.
 */
const i32 EDAM_RELATEDNOTES_PHRASE_SLOP_MAX = 16;


// ============================== Structures ===================================

/**
 * Structure returned by calls to createUser which can either hold a
 * successfully-created User or else can hold information about conflicts
 * which should be relayed to the user for resolution.
 *
 *<dl>
 * <dt>user: <dd>If the user creation was successful, this will hold the created
 *   user object.  If the user cannot be created due to a username and/or
 *   email conflict, this will not be set.
 *
 * <dt>usernameSuggestions: <dd>If the user cannot be created due to a
 *   conflicting username, this will be set.
 *   It may contain a list of suggested alternates.
 *
 * <dt>emailConflict: <dd>If true, then the user cannot be created with the
 *   requested email address because it was already in use.
 * </dl>
 */
struct CreateUserResult {
  1:  optional  Types.User user,
  2:  optional  list<string> usernameSuggestions,
  3:  required  bool emailConflict
}


/**
 * Structure used to represent an application  that was granted an API consume
 * key to access Evernote.
 *
 *<dl>
 * <dt>consumerKey: <dd>The OAuth consumer_key, which is used to identify the
 *   external application.
 *
 * <dt>consumerSecret: <dd>OAuth 'consumer_secret' that will be used to "sign"
 *   messages from the other application.
 *
 * <dt>permissions: <dd>This integer value holds a set of bits representing the
 *   access permissions of this Consumer.  The individual bit values are
 *   represented in the AuthenticationPermissions enumeration.
 *
 * <dt>callbackUrlBase: <dd>If the API key is used by a web application partner,
 *   then this may be used to specify the permitted callback URL on the partner
 *   site, or the initial prefix for the URL
 *
 * <dt>maxSessionDuration: <dd>If this is set, this specifies the longest
 *   session that may be requested by the holder of the API key.
 *
 * <dt>maxRequestsPerMinute: <dd>If this is set, this specifies the maximum
 *   number of requests permitted from the holder of this API key, per shard.
 *
 * <dt>organization: <dd>If set, this is the printable name of the Consumer
 *   organization (e.g. web service, etc.).
 * </dl>
 */
struct OAuthConsumer {
  1:  required  string consumerKey,
  2:  required  string consumerSecret,
  3:  required  i32 permissions,
  4:  optional  string callbackUrlBase,
  5:  optional  i64 maxSessionDuration,
  6:  optional  i32 maxRequestsPerMinute,
  7:  optional  string organization
}


/**
 * An OAuthSession represents the unique binding between an external OAuth
 * Consumer and a user account in Evernote.  Until the session expires, or is
 * revoked by the user, it may be used by the Consumer site to access the
 * user's account.
 *
 *<dl>
 * <dt>accessToken: <dd>The Evernote authentication token representing this
 *   session, which may be used to access the account via NoteStore and
 *   UserStore API calls.
 *
 * <dt>consumerKey: <dd>The OAuth consumer_key identifying the OAuth Consumer
 *   who holds this session.
 *
 * <dt>userId: <dd>The unique identifier of the account given access by this
 *   session.
 *
 * <dt>expires: <dd>The date and time when this session will no longer be
 *   usable to access the account.  After this, the user must re-approve
 *   access by this application.
 *
 * <dt>requestToken: <dd>If set, this is the OAuth request token that can be
 *   used by the consumer to retrieve the accessToken
 *
 * </dl>
 */
struct OAuthSession {
  1:  optional  string accessToken,
  2:  required  string consumerKey,
  3:  required  Types.UserID userId,
  4:  optional  Types.Timestamp expires,
  5:  optional  string requestToken
}

/**
 * Gift is a structure representing a user's attempt to purchase a gift of Evernote
 * for someone else.
 * <dl>
 * <dt> id: <dd>The unique auto-incramenting number of attempted gifts
 * <dt> userId: <dd>The user id of the purchaser
 * <dt> purchaseCode:<dd>A system generated unique string for the purchase. This
 * alone can be used to redeem the purchase.
 * <dt> fromEmail:<dd>The email address the user used to identifty themselves
 * <dt> fromName:<dd>The name the user used to identify themselves
 * <dt> toEmail:<dd>The email address of the recipient
 * <dt> toName:<dd>The name of the recipient
 * <dt> cardCode:<dd>An identify use to represent which card type the sender selected
 * <dt> commerceService:<dd>The service used to purchase the gift
 * <dt> orderNumber:<dd>The number used to trace back the purchase order
 * <dt> serviceSku:<dd> The SKU of the purchase (allowing multiple gift types)
 * <dt> message:<dd> The message sent to the recipient
 * <dt> recipient:<dd>When the purchase is redeemed, the user id of the redeemer is recorder
 * <dt> created:<dd>The date of the purchase request
 * <dt> purchased:<dd>Date/time the purchase was transacted
 * <dt> sent:<dd>Date/time the email was sent to the gift recipient
 * <dt> activated:<dd>Date/time the recipient upgraded their account to premium
 * <dt> updated:<dd>Date/time the gift record was changed
 * </dl>
 *
 */
struct Gift {
  1:  optional i32    id;
  2:  optional Types.UserID userId,
  3:  optional string purchaseCode,
  4:  optional string fromEmail,
  5:  optional string fromName,
  6:  optional string toEmail,
  7:  optional string toName,
  8:  optional string cardCode,
  9:  optional string commerceService,
  10: optional string orderNumber,
  11: optional string serviceSku,
  12: optional string message,
  13: optional Types.UserID recipient,
  14: optional Types.Timestamp created,
  15: optional Types.Timestamp purchased,
  16: optional Types.Timestamp sent,
  17: optional Types.Timestamp activated,
  18: optional Types.Timestamp updated
}


struct AccountingDetail {
 1:   required  i64             id,
 2:   required  Types.UserID    userId,
 3:   required  string          commerceService,
 4:   required  string          orderNumber,
 5:   required  Types.Timestamp created,
 6:   required  string          item,
 7:   required  i32             quantity,
 8:   required  i32             unitPrice,
 9:   optional  Types.Timestamp charged,
10:   optional  string          currency,
11:	  optional  string			country,
12:	  optional  string          state,
13:   optional  string          postalCode,
14:   optional  i32             unitTax
}

struct LocalizedResource {
 1:   required Types.Resource   resource,
 2:   required string           locale,
 3:   optional byte             priority
 4:   optional Types.Timestamp  queuedTime
}

struct AdFilter {
  1:  required  i32 adId,
  2:  optional  i32 filterId,
  3:  required  string clientProperty,
  4:  required  string value,
  5:  required  AdFilterType filterType
}

struct AdInternal {
  1:  required  Types.Ad ad,
  2:  required  i32 advertiserId,
  3:  optional  double weight,
  4:  optional  Types.Timestamp  endDate,
  5:  optional  string networkAdId,
  6:  optional  string language,
  7:  optional  list<AdFilter> filters
}

struct Advertiser {
  1:  required  i32 id,
  2:  required  string name,
  3:  optional  double weight,
  4:  optional  string category,
  5:  optional  string network,
  6:  optional  string networkAdvertiserId
}

struct ApiKey {
  1:  required  string 			consumerKey,
  2:  optional  string 			consumerSecret,
  3:  optional  string 			callbackUrlBase,
  4:  required  i32    			permissions,
  5:  optional  i64    			maxSessionDuration,
  6:  optional  i32			    maxRequestsPerMinute,
  7:  optional  string 			organization,
  8:  optional  string 			contactName,
  9:  optional  string 			contactEmail,
  10: required  Types.Timestamp created,
  11: optional  string      notificationUrl,
  12: optional  string      notificationFilter
}

/**
 * Represents an action performed by a Support or Admin person through the admin
 * UI with a particular account.
 */
struct SupportAction {
  1:  required  i32 targetUserId,
  2:  required  i32 adminUserId,
  3:  optional  Types.Timestamp created,
  4:  optional  string caseId,
  5:  optional  string changes
}

struct Sponsor {
  1: optional i32 sponsorId,
  2: required string name,
  3: required string uri,
  4: required i16 accountLimit,
  5: required bool approvalRequired,
  6: optional string welcome,
  7: required Types.Timestamp createdDate,
  8: optional Types.Timestamp updatedDate,
  9: required BillingType billingType,
  10: optional string displayName,
  11: optional string emailDomain
}

struct SponsorImage {
  1: required i32 sponsorId,
  2: optional binary image,
  3: optional string name
}

struct SponsorMember {
 1: required string username,
 2: required i32 userId,
 3: required string fullName,
 4: required string email,
 5: required SponsorRole role,
 6: required SponsorUserStatus status,
 7: required i32 sponsorId,
 8: optional Types.Timestamp createdDate,
 9: optional Types.Timestamp activatedDate,
 10: optional Types.Timestamp deactivatedDate
}

/**
 * This enumeration defines the possible sort ordering for notes when
 * they are returned from a search result.
 */
enum SponsorMemberSortOrder {
  SPONSOR_MEMBER_CREATED = 1,
  SPONSOR_MEMBER_ROLE = 2,
  SPONSOR_MEMBER_STATUS = 3,
  SPONSOR_MEMBER_ACTIVATED = 4,
  SPONSOR_MEMBER_DEACTIVATED = 5,
  SPONSOR_MEMBER_USER_ID = 6,
  SPONSOR_MEMBER_USERNAME = 7,
  SPONSOR_MEMBER_EMAIL = 8
}

/**
 * A list of criteria that are used to indicate which notes are desired from
 * the account.  This is used in queries to the NoteStore to determine
 * which notes should be retrieved.
 *
 *<dl>
 * <dt>order</dt>
 *   <dd>
 *   The SponsorMemberSortOrder value indicating what criterion should be
 *   used to sort the results of the filter.
 *   </dd>
 *
 * <dt>ascending</dt>
 *   <dd>
 *   If true, the results will be ascending in the requested
 *   sort order.  If false, the results will be descending.
 *   </dd>
 *
 * <dt>words</dt>
 *   <dd>
 *   The string query containing keywords to match, if present.
 *   </dd>
 *
 * <dt>roles</dt>
 *   <dd>
 *   An unordered set of SponsorRole values indicating which roles the results should be restricted to.
 *   </dd>
 *
 * <dt>status</dt>
 *   <dd>
 *   An unordered set of SponsorUserStatus values indicating which statuses the results should be restricted to.
 *   </dd>
 *
 * <dt>deactivated</dt>
 *   <dd>
 *   If true, the results will include deactivated users
 *   </dd>
 * </dl>
 */
struct SponsorMemberFilter {
  1: optional  i32 order,  // Should be one of the SponsorMemberSortOrder values
  2: optional  bool ascending,
  3: optional  string words,
  4: optional  set<SponsorRole> roles,
  5: optional  set<SponsorUserStatus> statuses,
  6: optional  list<Types.UserID> userIds
}

struct SponsorResult {
  1: required i32 total,
  2: optional list<SponsorMember> members,
  3: optional i32 availableTotal
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * An enumerate of fields that are known to the related notes functionality.
 *
 * <dl>
 *   <dt>IRNF_CREATED</dt>
 *     <dd>The <tt>created</tt> timestamp field of a note.
 *     </dd>
 * </dl>
 */
enum InternalRelatedNotesFields {
  IRNF_CREATED   = 1,
  IRNF_PLACENAME = 3
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * Specifies required-status for the identified query clause/term.
 *
 * <dl>
 *   <dt>IRNR_MUST</dt>
 *     <dd>The clause is required to match in order for the note to be
 *         considered for inclusion in the result set.  No matter how
 *         high the score may be, failing a MUST condition will keep the
 *         note from being included.
 *     </dd>
 *   <dt>IRNR_MUST_NOT</dt>
 *     <dd>The clause must not match in order for the note to be considered
 *         for inclusion in the result set.  No matter how high the score
 *         may be, if the clause or condition is satisfied, the note will
 *         not be considered in the result set.  
 *     </dd>
 *   <dt>IRNR_SHOULD</dt>
 *     <dd>Indicates that matching, or satisfying, the clause is optional
 *         in regards to inclusion in the result set.
 *     </dd>
 * </dl>  
 */
enum InternalRelatedNotesRequirement {
  IRNR_MUST     = 1,
  IRNR_MUST_NOT = 2,
  IRNR_SHOULD   = 3
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * Allows the specification of a note to be used as an example note for
 * "more like this" functionality.
 * 
 * <dl>
 *   <dt>guid</dt>
 *     <dd>The GUID of the note to be used as an example.</dd>
 *   <dt>maxQueryTerms</dt>
 *     <dd>The algorithm will pick the top terms from the note to include in
 *         the query for related notes, based on term frequency.  The server
 *         imposes a maximum of EDAM_RELATEDNOTES_MAX_TERMS_FROM_EXAMPLE,
 *         but you may choose fewer terms if you wish.
 *     </dd>
 *   <dt>minTermFrequency</dt>
 *     <dd>A term must occur at least this many times in order to be included
 *         in the query for related notes.
 *     </dd>
 *   <dt></dt>
 *     <dd>An optional boost factor that specifies the relative importance
 *         of our example note's contribution to the overall score of the
 *         result notes.
 *     </dd>
 *   <dt>requirement</dt>
 *     <dd>See InternalRelatedNotesRequirement for details.
 *     </dd>
 * </dl>
 */
struct InternalRelatedNotesExample {
  1: required Types.Guid guid,
  2: optional i32 maxQueryTerms,
  3: optional i32 minTermFrequency,  
  4: optional double scoreBoost,
  5: optional InternalRelatedNotesRequirement requirement 
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * ... need better documentation, but similar to RelatedNotesExample except
 *     we have a block of plain text to use as our example.  The maximum
 *     length of this text is EDAM_RELATEDNOTES_TEXT_LEN_MAX.   
 */
struct InternalRelatedNotesExampleText {
  1: optional string text,
  2: optional i32 maxQueryTerms,
  3: optional i32 minTermFrequency,
  4: optional double scoreBoost,
  5: optional InternalRelatedNotesRequirement requirement
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * NOT YET IMPLEMENTED, IGNORE FOR NOW.
 */
struct InternalRelatedNotesTerms {
  1: required string text,   // if slop is set, will be phrase,
                             // else if ends with *, will be prefix
  2: optional InternalRelatedNotesFields field,
  3: optional double scoreBoost,
  4: optional i32 slop,
  5: optional InternalRelatedNotesRequirement requirement
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * Specifies a timestamp range on a particular field as part of the search
 * for related notes.  The range may have either the min or max undefined
 * (effectively negative and positive infinity), but not both.
 * 
 * <dl>
 *   <dt>field</dt>
 *     <dd>Identifies the field on which we are applying a range.
 *     </dd>
 *   <dt>minInclusive</dt>
 *     <dd>The minimum value allowed, inclusive.
 *     </dd>
 *   <dt>maxInclusive</dt>
 *     <dd>The maximum value allowed, inclusive.
 *     </dd>
 *   <dt>scoreBoost</dt>
 *     <dd>An optional boost factor that specifies the relative importance of
 *         the range specification to the overall score of the result notes.
 *     </dd>
 *   <dt>requirement</dt>
 *     <dd>See InternalRelatedNotesRequirement for details.
 *     </dd>
 * </dl>
 */
struct InternalRelatedNotesTimestampRange {
  1: required InternalRelatedNotesFields field,
  2: optional Types.Timestamp minInclusive,
  3: optional Types.Timestamp maxInclusive,
  4: optional double scoreBoost,
  5: optional InternalRelatedNotesRequirement requirement
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 * <b>NOTE!! CURRENT IMPLEMENTATION DOES NOT TAKE EDGE CASES INTO ACCOUNT!!</b> 
 * 
 * Specifies a geographic range, between given latitudes and longitudes,
 * to compare against a note's latitude and longitude values.
 *
 * <dl>
 *   <dt>latitudeCenter</dt>
 *     <dd>The latitude that is at the center of the region.
 *     </dd>
 *   <dt>latitudePad</dt>
 *     <dd>The number of miles to pad out the region on each side of the
 *         latitude.  A fractional number of miles is possible.
 *     </dd>
 *   <dt>longitudeCenter</dt>
 *     <dd>The longitude that is at the center of the region.
 *     </dd>
 *   <dt>longitudePad</dt>
 *     <dd>The number of miles to pad out the region on each side of the
 *         longitude.  A fractional number of miles is possible.
 *     </dd>
 *   <dt>scoreBoost</dt>
 *     <dd>An optional boost factor that specifies the relative importance of
 *         the range specification to the overall score of the result notes.
 *     </dd>
 *   <dt>requirement</dt>
 *     <dd>See InternalRelatedNotesRequirement for details.
 *     </dd>
 * </dl>   
 * 
 */
struct InternalRelatedNotesGeographicRange {
  1: optional double latitudeCenter,
  2: optional double latitudePad,
  3: optional double longitudeCenter,
  4: optional double longitudePad,
  5: optional double scoreBoost,
  6: optional InternalRelatedNotesRequirement requirement
}

/**
 * <b>THIS IS NOT YET A PRODUCTION API METHOD!</b>
 *
 * The structure that captures a query for related notes.  See the various
 * structure definitions for details.  Everything is optional individually,
 * but you must specify some constraint for the query.  The time zone
 * can be specified for timestamp-related queries and has semantics similar
 * to findNotes().   
 */
struct InternalRelatedNotesQuery {
  1: optional InternalRelatedNotesExample exampleNote,
  2: optional InternalRelatedNotesExampleText exampleText,                                               
  3: optional InternalRelatedNotesGeographicRange geoRange,
  4: optional list<InternalRelatedNotesTerms> terms,
  5: optional list<InternalRelatedNotesTimestampRange> timestampRanges,
  6: optional string timeZone
/* minimum number of "should" clauses to be added... */
}

/**
 * THIS IS **NOT** HOW IT WILL BE IN PRODUCTION ... just grabbing extra data
 * to understand how the scoring works for now.
 * 
 * Represents a single hit, or result, from the search for related notes.
 *
 * <dl>
 *   <dt>note</dt>
 *     <dd>Will be the note for the search result, filled in similar to
 *         how findNotes() fills in notes. 
 *     </dd>
 *   <dt>explanation</dt>
 *     <dd>If present, will be an HTML-formatted explanation of how the
 *         result was scored, useful only for tuning and debugging.
 *     </dd>
 *   <dt>score</dt>
 *     <dd>The score for the note in the result set, and which may not
 *         be present.
 *     </dd>
 * </dl>
 */
struct InternalRelatedNotesHit {
  1: optional Types.Note note,
  2: optional string explanation,
  3: optional double score
}

/**
 * THIS IS **NOT** HOW IT WILL BE IN PRODUCTION, grabbing extra data for now.
 * 
 * The result of a related notes search.
 *
 * <dl>
 *   <dt>hits</dt>
 *     <dd>The returned list of results, sorted by relevance.
 *     </dd>
 *   <dt>totalNotes</dt>
 *     <dd>The total number of results that matched the query.  Note that
 *         these include the results returned in "hits" as well as notes
 *         that did not score high enough to make it into "hits".
 *     </dd>
 *   <dt>searchTime</dt>
 *     <dd>The time spent in the search database for the query, which is
 *         presented only for debugging and tuning purposes.
 *     </dd>
 *   <dt>query</dt>
 *     <dd>The query string that was formed and used against the native
 *         search engine of the service.
 *     </dd>
 * </dl>
 */
struct InternalRelatedNotesResult {
  1: optional list<InternalRelatedNotesHit> hits,
  2: optional i32 totalNotes,
  3: optional Types.Timestamp searchTime,
  4: optional string query  
}

// ============================== Service ===================================

/**
 * Service:  EverNoteInternal
 *
 * Trusted operations that may only be performed by internal components of
 * the EverNote Service.
 */
service EverNoteInternal {

  /**
   * Tries to make a new account in the system using the fields provided in the
   * provided structure.  This account will be created on the NoteStore
   * shard which receives the message, so this should be sent to the
   * shard that is appropriate for creating new accounts.
   * If the requested username and/or email address is taken, the result
   * will not contain a User, and will contain information about the source
   * of the conflict.
   *
   * @param userTemplate
   *   A template of the user data to store in the service
   *   account.  The id, password and shardId fields in this template will
   *   be ignored.  Any attributes will be set in the account.
   *
   * @param password
   *   The user's desired password.
   *
   * @param firstNotebookName
   *   The account will be created with an initial notebook
   *   using this name, if provided, or a default value if this is absent.
   */
  CreateUserResult createUser(1: Types.User userTemplate,
                              2: string password,
                              3: string firstNotebookName)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException),

  /**
   * Returns the user record of the user with the provided UserID, or
   * throws EDAMNotFoundException if not found
   */
  Types.User getUserByID(1: Types.UserID userID)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the user record of the user with the provided username, or
   * throws EDAMNotFoundException if not found
   */
  Types.User getUserByName(1: string username)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the user record of the user with the provided email, or
   * throws EDAMNotFoundException if not found
   */
  Types.User getUserByEmail(1: string email)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the user record for the user who has the provided
   * incomingEmailAddress attribute set, or throws EDAMNotFoundException
   * if this address was not found.
   */
  Types.User getUserByIncomingEmailAddress(1: string email)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * If an Evernote user with the provided twitter ID has chosen to link
   * Twitter to their Evernote account, then this will return the linked User.
   * Otherwise, throws EDAMNotFoundException
   */
  Types.User getUserByTwitterId(1: string twitterId)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Updates the state of a user account.
   * This will change the fields on User, and on UserAttributes if set, but
   * will not modify any of the fields on the Accounting structure, if provided.
   */
  void updateUser(1: Types.User user)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Updates the accounting information for a user.
   */
  void updateAccounting(1: Types.UserID userID,
                        2: Types.Accounting accounting)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),
  /**
   * Updates the SKU and Commerce Service associated with a user.
   */
  void updateAccountingSKUAndCommerce(1: Types.UserID userId,
						  			  2: string skuCode,
						  			  3: string commerceService)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Sets the password for the user to the provided value.
   */
  void setPassword(1: Types.UserID userID,
  	               2: string password)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the hash of the user's password from the database.
   */
  string getPasswordHash(1: Types.UserID userID)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Forges an authentication token that allows access to a particular
   * user account.
   *
   * @param duration
   *   the number of milliseconds (from the server's current time)
   *   that the authentication token will be usable to access the account.
   *   If the parameter is 0 or less, the default authentication token
   *   duration will be used.
   *
   * @param permissions
   *   a list of the access rights that should be granted to the authentication
   *   token.  The bearer will only be able to perform operations that are
   *   included in this set.  Attempts to invoke other operations will result
   *   in EDAMUserException with PERMISSION_DENIED.
   */
  string createAuthenticationToken(1: Types.UserID id,
                                  2: i64 duration,
                                  3: list<AuthenticationPermission> permissions)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Permanently revokes an existing authentication token so that it may no
   * longer be used to access the corresponding user account.
   */
  void revokeAuthenticationToken(1: string token)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns one User from this particular shard
   */
  Types.User findAnyShardUser()
    throws (1: Errors.EDAMNotFoundException notFoundException,
            2: Errors.EDAMSystemException systemException),

  /**
   * Asks for a resource to recognize.  Returns an appropriate resource or
   * throws a EDAMErrorCode.NOT_FOUND EDAMUserException.
   * @param mimeTypes  the list of MIME types for resources that are requested
   * @param leaseLength  the duration of the lease, in seconds
   * @param minimumPriority  only return resources of this priority level or
   *   higher
   */
  LocalizedResource leaseLocalizedResourceForRecognition(1: list<string> mimeTypes,
                                                         2: i32 leaseLength,
                                                         3: i32 minimumPriority)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Submits an indexed recognition result to the server using the GUID.
   * @param resourceGuid the resourceGuid with the associated index and words
   * @param index the recoIndex XML document containing the data found within
   * resource
   */
  void submitRecognitionIndex(1: string resourceGuid,
  	                          2: binary index)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Submits an alternate data representation for the resource, which may be
   * more appropriate for searching or display.
   * @param resourceGuid the resourceGuid with the associated index and words
   * @param index the bytes of the alternate resource data form
   * resource
   */
  void submitResourceAlternateData(1: string resourceGuid,
                                   2: binary alternateData)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the thumbnail to display for the note with the provided GUID,
   * or throws an EDAMNotFoundException if no such thumbnail is cached on the
   * service.
   */
  binary getNoteThumbnail(1: string noteGuid)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Tells the service to cache the provided thumbnail for the note with
   * the indicated GUID
   */
  void setNoteThumbnail(1: string noteGuid,
                        2: binary thumbnail)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the cached "square" format thumbnail for the provided note,
   * or throws EDAMNotFoundException if none is stored
   */
  binary getNoteSquareThumb(1: string noteGuid)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Stores the cached "square" format thumbnail for the provided note
   */
  void setNoteSquareThumb(1: string noteGuid,
                        2: binary thumbnail)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the cached "square" format thumbnail for the provided resource,
   * or throws EDAMNotFoundException if none is stored
   */
  binary getResourceSquareThumb(1: string guid)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Stores the cached "square" format thumbnail for the provided resource
   */
  void setResourceSquareThumb(1: string guid,
                        2: binary thumbnail)
    throws (1: Errors.EDAMSystemException systemException),


  /**
   * Gives the shard a chance to do any initialization activities
   */
  void start(),

  /**
   * Tells the shard to stop and prepare for exit
   */
  void stop(),

  /**
   * Crawls through the user's account to check for any problems with data
   * format (e.g. notes not in proper ENML format) or integrity (e.g. a
   * resource file not found on disk) and returns a plaintext report of the
   * problems.  If no problems were found, this will return an empty string.
   * <p/>
   * Needless to say, this is a horribly expensive operation, so should be
   * used wisely.
   */
  string auditUserAccount(1: Types.UserID userID)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the user record who has a corresponding premium
   * commerce order.
   */
  Types.User getUserByOrderNumber(1: string commerceService,
                                  2: string orderNumber)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the user record who has an outstanding commerce order which should
   * be evaluated either to de-active the premium account or charge the account.
   */
  Types.User leaseUserWithOutstandingOrder(1: Types.Timestamp lockOrderUntil)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns a locked user record which is set to expire between startDate
   * and endDate.
   * Account should be active, premium, in good standing, and have a
   * non-recurring SKU.
   */
  Types.User leaseUserWithExpiringSubscription(1: Types.Timestamp startDate,
                                               2: Types.Timestamp endDate,
                                               3: Types.Timestamp lockOrderUntil)
    throws (1: Errors.EDAMNotFoundException notFoundException,
            2: Errors.EDAMSystemException systemException),

  /**
   * Returns the user record who has a corresponding referral proof entry
   */
  Types.User getUserByReferralProof(1: string referralProof)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException),


  /**
   * Sets the lease date on a user to allow rescheduling recurring payments if
   * problems occurred during billing.
   */
  void updateUserLeaseOrder(1: Types.UserID userID,
            2: Types.Timestamp lockOrderUntil)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

// ====================== Commerce ==========================================

  /**
   * Modifies a user's account so that it is configured to provide a premium
   * service. The Premium service is not activated but the subscription number
   * is recorded.
   */
   void startPremiumService(1: Types.UserID userID,
            2: string skuCode,
            3: string commerceService, 4: string subscriptionOrderNumber)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),
  /**
   * Updates the premium account information to indicate the a recurring payment
   * charge will be attempted and the order number associated with the recurring
   * payment.
   */
  void startPremiumRecurringPayment(1: Types.UserID userID, 2: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Flags that a payment has been made. This will re-activate a premium account
   * if it had become inactive or simply advance the nextPayment due date.
   */
  void recurringPaymentMade(1: Types.UserID userID, 2: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Flags that a payment has been made. This will advance the nextPayment due date.
   */
  void extendedPaymentMade(1: Types.UserID userID, 2: string sku,
    3: string commerceService, 4: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * This will advance the nextPayment due date based on the sku. A purchase will
   * not be recorded and commerce account information will not be updated.
   * This will allow activation of gifts for users who already have premium
   * account.
   */
  void extendedNextPayment(1: Types.UserID userID, 2: string sku)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),


  /**
   * Sets the premium status to failed downgrading the upload quota and the
   * account permissions.
   */
  void recurringPaymentFailed(1: Types.UserID userID, 2: string reason)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Downgrades the account setting the next upload quota to the free subscription
   * level.
   */
  void cancelPremiumService(1: Types.UserID userID)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),
 /**
  *
  */
  void recurringPaymentRequested(1: Types.UserID userID, 2: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Creates a gift in an initial state: unpaid for, but completely constructed
   */
  void createGiftPurchase(1: Gift gift)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Lists the gifts associated with the purchase
   */
  list<Gift> listGifts(1: string commerceService, 2: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException),


  /**
   * Lists the gifts associated with the user
   */
  list<Gift> listPurchasedGifts(1: Types.UserID userID, 2: i32 offset, 3: i32 maxResults)
     throws (1: Errors.EDAMSystemException systemException,
     2: Errors.EDAMNotFoundException notFoundException),

  /**
   * get the gift count with the user
   */
  i32 purchasedGiftsSize(1: Types.UserID userID)
     throws (1: Errors.EDAMSystemException systemException,
     2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Lists redeemed gifts by recipient
   */
  list<Gift> listRedeemedGiftsByRecipient(1: Types.UserID userID)
     throws (1: Errors.EDAMSystemException systemException,
     2: Errors.EDAMNotFoundException notFoundException),

 /**
   * Returns a gift based on a promotion code
   */
  Gift getGift(1: string purchaseCode)
     throws (1: Errors.EDAMSystemException systemException,
     2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Updates the gift allowing the caller to define the states (paid, sent, redeemed)
   */
  void updateGift(1: Gift gift)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),


  /**
   * Creates a list of accounting details for purchase transaction
   */
  void createAccountingDetails(1: list<AccountingDetail> accountingDetails)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Lists the accounting details for a purchase
   */
  list<AccountingDetail> listAccountingDetailsByOrder(1: string commerceService, 2: string orderNumber)
     throws (1: Errors.EDAMSystemException systemException),


  /**
   * Lists the accounting details associated with the user
   */
  list<AccountingDetail> listAccountingDetails(1: Types.UserID userID)
     throws (1: Errors.EDAMSystemException systemException,
     2: Errors.EDAMNotFoundException notFoundException),


  /**
   * Updates the accounting details with transaction complete dated
   */
  void updateAccountingDetails(1: list<AccountingDetail> accountingDetails)
     throws (1: Errors.EDAMSystemException systemException,
             2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Updates the accounting details for members of Sponsored Accounts
   */
  SponsorMember updateSponsorMemberAccounting(1: SponsorMember member)
      throws (1: Errors.EDAMUserException userException,
      		  2: Errors.EDAMSystemException systemException,
      		  3: Errors.EDAMNotFoundException notFoundException),

  // ====================== OAuth ============================================

  /**
   * Looks for the OAuth Consumer on this service identified via the provided
   * OAuth consumer_key.
   */
  OAuthConsumer getOAuthConsumer(1: string consumerKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the list of long-lived OAuth sessions associated with the User
   */
  list<OAuthSession> getOAuthSessions(1: Types.UserID userId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Returns the OAuthSession that was created using the indicated requestToken,
   * or throws EDAMNotFoundException
   */
  OAuthSession getOAuthSessionByRequestToken(1: string requestToken)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Makes a new OAuthSession based on the information in the provided template,
   * and then returns the accessKey for the created session.
   */
  string createOAuthSession(1: OAuthSession session)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * The provided session information will be saved into the database.  If the
   * included accessToken is new, this will create a new record, otherwise this
   * will update the OAuthSession with that accessToken
   */
  void updateOAuthSession(1: OAuthSession session)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  // ====================== Advertising =======================================

  /**
   * Retrieves all ads in the database. This will not include the Ad.image
   * field due to possible total sizes. To access the image for an Ad,
   * invoke getAd()
   */
  list<AdInternal> getAds()
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the ad with the provided id
   */
  AdInternal getAd(1: i32 adId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Creates an ad based on the provided specification. This will return the
   * representation of the created ad, but without the Ad.image field set,
   * reduce the overall size.
   */
  AdInternal createAd(1: AdInternal adInternal)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Updates the fields of the stored ad based on the provided template
   */
  void updateAd(1: AdInternal adInternal)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Removes the ad with the given identifier from the database
   */
  void deleteAd(1: i32 adId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),
  /**
   * Returns the advertisers in the database.  If 'useCache' is false, this
   * will force a direct DB hit.
   */
  list<Advertiser> getAdvertisers(1: bool useCache)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the advertiser with the given identifier
   */
  Advertiser getAdvertiser(1: i32 advertiserId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Creates a new advertiser in the database
   */
  Advertiser createAdvertiser(1: Advertiser advertiser)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Updates the stored field on the advertiser matching the provided template
   */
  void updateAdvertiser(1: Advertiser advertiser)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Removes the provided advertiser from the database.
   */
  void deleteAdvertiser(1: i32 advertiserId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException)

  /**
   * Creates a filter for an existing ad
   */
  AdFilter createAdFilter(1: AdFilter filter)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException)

  /**
   * Removes the filter with the provided ID from the ad
   */
  void deleteAdFilter(1: i32 adId, 2: i32 filterId)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException)

  /**
   * Computes a weighted random score for each ad in the system and updates
   * the database to reflect the new scores.
   */
  void randomScoreAds()
    throws (1: Errors.EDAMSystemException systemException)

  /**
   * Assign a user to a share. Exception is thrown if it's already used.
   */
  Types.SharedNotebook assignUserToShare(1: i64 shareId, 2: string username)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException,
            3: Errors.EDAMUserException userException),

  Types.SharedNotebook getSharedNotebook(1: string shareKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Return the shard ID of this shard.
   */
  string getShardId()
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Specifies that a particular task should be performed within the account
   * of a user.  This call may be made from any shard, and the task will be
   * performed on the user's own shard.
   */
  void scheduleUserTask(1: i32 userId,
                        2: UserTaskType taskType,
                        3: Types.Timestamp after)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * List the API keys in the database.  If 'useCache' is false, this
   * will force a direct DB hit.
   */
  list<ApiKey> listApiKeys(1: bool useCache)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Get the API key with the given identifier.
   */
  ApiKey getApiKey(1: string consumerKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Find API keys for given organization. Searches by substring,
   * case-insensitive. Zero or more keys are returned.
   */
  list<ApiKey> findApiKeysByOrganization(1: string organizationSubstring)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Create a new API key in the database.
   */
  ApiKey createApiKey(1: ApiKey apiKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMUserException userException),

  /**
   * Update the stored field on the API key matching the provided template.
   */
  void updateApiKey(1: ApiKey apiKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMUserException userException,
            3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Remove the provided API key from the database.
   */
  void deleteApiKey(1: string consumerKey)
    throws (1: Errors.EDAMSystemException systemException,
            2: Errors.EDAMNotFoundException notFoundException),

  /**
   * Identical to NoteStore.createNote() except that the user's monthly upload
   * allowance is not "charged" for the created note.
   */
  Types.Note createNoteWithoutQuota(1: string authenticationToken,
                        2: Types.Note note)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMSystemException systemException,
            3: Errors.EDAMNotFoundException notFoundException)

  /**
   * Records an action performed by a Support person via the admin UI
   */
  SupportAction createSupportAction(1: SupportAction action)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the most recent actions performed on a user account.
   */
  list<SupportAction> findSupportActionsByTargetUser(1: i32 targetUserId,
                                                     2: i32 offset,
                                                     3: i32 limit)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Returns the most recent actions performed by a particular administrator.
   */
  list<SupportAction> findSupportActionsByAdminUser(1: i32 adminUserId,
                                                    2: i32 offset,
                                                    3: i32 limit)
    throws (1: Errors.EDAMSystemException systemException),

  /**
   * Create a new sponsored group associated with the specified user id.
   * <dl>
   * <dt>userId</dt>
   * <dd>owner user id</dd>
   * <dt>sponsor</dt>
   * <dd>Sponsor meta data</dd>
   * </dl>
   */
  Sponsor createSponsor(1: Types.UserID userId,
   						2: Sponsor sponsor)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Update the sponsor meta data.
   * <dl>
   * <dt>sponsor</dt>
   * <dd>Sponsor to update</dd>
   * </dl>
   */
  Sponsor updateSponsor(1: Sponsor sponsor)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Update an image associated with a sponsored group.
   * <dl>
   * <dt>image</dt>
   * <dd>Image data associated with a sponsored group</dd>
   * </dl>
   */
  void updateSponsorImage(1: SponsorImage image)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Get an image associated with a sponsored group.
   * <dl>
   * <dt>sponsorId</dt><dd>id associated with sponsored group.</dd>
   * </dl>
   */
  SponsorImage getSponsorImage(1: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Add a member to a Sponsored group.
   * <dl>
   * <dt>userId</dt><dd>User id of new member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  SponsorMember addSponsorMember(1: Types.UserID userId,
                                 2: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Remove a member from a sponsored group.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  SponsorMember removeSponsorMember(1: Types.UserID userId,
                                    2: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Search for sponsored members using a filter.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * <dt>filter</dt><dd>Criteria for search</dd>
   * <dt>offset</dt><dd>offset from start of search results</dd>
   * <dt>maxResults</dt><dd>max results to return</dd>
   * </dl>
   */
  SponsorResult findSponsorMembers(1: i32 sponsorId,
                       2: SponsorMemberFilter filter
                       3: i16 offset,
                       4: i16 maxResults)
      throws (1: Errors.EDAMUserException userException,
          2: Errors.EDAMSystemException systemException,
          3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Get the owner for a specific sponsored group.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  SponsorMember getSponsorOwner(1: i32 sponsorId)
      throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Update the SponsorUserStatus for a specific user.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * <dt>status</dt><dd>new status of user</dd>
   * </dl>
   */
  SponsorMember setSponsorStatus(1: Types.UserID userId,
                                 2: i32 sponsorId,
                                 3: SponsorUserStatus status)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Update the SponsorRole for a specific user.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * <dt>role</dt><dd>new role of user</dd>
   * </dl>
   */
  SponsorMember setSponsorRole(1: Types.UserID userId,
                                 2: i32 sponsorId,
                                 3: SponsorRole role)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Find the currently active sponsor membership for the specified user id.
   * User's may be members of more than one sponsored account,
   * but only one of those memberships may be active at any time.
   * This call ignores any memberships that belong to canceled groups.
   * We throw a NotFoundException if the member is not currently part of any
   * active groups.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * </dl>
   */
  SponsorMember getActiveSponsorMemberByUserId(1: Types.UserID userId)
    throws(1: Errors.EDAMNotFoundException notFoundException,
    	   2: Errors.EDAMUserException userException,
    	   3: Errors.EDAMSystemException systemException),

  /**
   * This finds a specific sponsor member entry associated with the user id
   * and sponsor id.  This is used when we are searching for canceled
   * sponsored group member information.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  SponsorMember getSponsorMemberByUserIdAndSponsorId(1: Types.UserID userId,
  													 2: i32 sponsorId)
    throws(1: Errors.EDAMNotFoundException notFoundException,
    	   2: Errors.EDAMUserException userException,
    	   3: Errors.EDAMSystemException systemException),

  /**
   * Find the sponsor for a specific sponsor id.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  Sponsor getSponsorById(1: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Find the sponsor for a specific sponsor uri.
   * Used by the group welcome pages.  Uri's are guaranteed unique by the sponsor schema.
   * <dl>
   * <dt>uri</dt><dd>unique postfix path of the sponsored group</dd>
   * </dl>
   */
  Sponsor getSponsorByUri(1: string uri)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Find the image associated with a specific sponsor uri.
   * Used by the group welcome pages.  Uri's are guaranteed unique by the sponsor schema.
   * <dl>
   * <dt>uri</dt><dd>unique postfix path of the sponsored group</dd>
   * </dl>
   */
  SponsorImage getSponsorImageByUri(1: string uri)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  bool hasSponsorImage(1: string uri),

  /**
   * Get the SponsorUserStatus associated with a specific user and sponsor.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  SponsorUserStatus getSponsorUserStatus(1: Types.UserID userId,
  										 2: i32 sponsorId)
  	throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Return the number of active members for a specific sponsored group.
   * Active is defined as members who have a status of either Active or
   * Deactivating Pending.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  i32 getSponsorUserActiveCount(1: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Return the number of pending members for a specific sponsored group.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  i32 getSponsorUserPendingCount(1: i32 sponsorId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Return the number of members for a specific sponsored group
   * based on search criteria.
   * <dl>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  i32 getSponsorUserCount(1: i32 sponsorId,
                      2: SponsorMemberFilter filter)
    throws (1: Errors.EDAMUserException userException,
          2: Errors.EDAMSystemException systemException,
          3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Return the group of canceled sponsored groups associated with this user id.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * </dl>
   */
  list<Sponsor> listCanceledSponsorsByOwner(1: Types.UserID userId)
    throws (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Remove a sponsored group.  This is destructive, and also removes the sponsored users too.
   * It is currently only used by the reactivation code.
   * <dl>
   * <dt>userId</dt><dd>User id of member</dd>
   * <dt>sponsorId</dt><dd>id of sponsored group</dd>
   * </dl>
   */
  bool deleteSponsor(1: Types.UserID userId, 2: i32 sponsorId)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Send an email template to the specified user.
   * To aid portability, we are deliberately not passing parameters to
   * the backend.
   * <dl>
   * <dt>userId</dt><dd>user id of the recipient</dd>
   * <dt>prefix</dt><dd>prefix for the email template to send</dd>
   * </dl>
   */
  bool emailTemplate(1: Types.UserID userId, 2: string prefix)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),

  /**
   * Get an Internal User Attribute for the specified user.
   *
   * <dl>
   * <dt>userID</dt><dd>User id to check</dd>
   * <dt>type</dt><dd>type for value to get</dd>
   * </dl>
   */
  string getInternalUserAttribute(1: Types.UserID userID, 2: InternalUserAttributeType type)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException)
    	    
  /**
   * Get all Internal User Attributes for the specified user.
   *
   * <dl>
   * <dt>userID</dt><dd>User id to check</dd>
   * </dl>
   */
  map<InternalUserAttributeType, string> getInternalUserAttributes(1: Types.UserID userID)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException)
    	    
  /**
   * Set an Internal User Attribute.
   *
   * <dl>
   * <dt>userID</dt><dd>User id to check</dd>
   * <dt>key</dt><dd>type for value to set</dd>
   * <dt>type</dt><dd>value to set</dd>
   * </dl>
   */
  void setInternalUserAttribute(1: Types.UserID userID, 2: InternalUserAttributeType type, 3: string value)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),
    	    
  /**
   * Delete an Internal User Attribute.
   *
   * <dl>
   * <dt>userID</dt><dd>User id to check</dd>
   * <dt>type</dt><dd>type for value to delete</dd>
   * </dl>
   * 
   * If removing a key that does not exist, no exception will be thrown.  This would require a read before the delete and is wasteful.
   */
  void deleteInternalUserAttribute(1: Types.UserID userID, 2: InternalUserAttributeType type)
   throws  (1: Errors.EDAMUserException userException,
    	    2: Errors.EDAMSystemException systemException,
    	    3: Errors.EDAMNotFoundException notFoundException),
    	                                                      
  /**
   * <b>NOTE A PRODUCTION API, INCLUDED FOR INVESTIGATION ONLY</b>.
   *   
   * Many fields used for debugging won't be present in the production API.
   *
   * @param query
   *   The query, or criteria, to use for finding related notes.
   *
   * @param maxNotes
   *   The most notes to return in this query.  The service will return a set
   *   of notes that is no larger than this number, but may return fewer notes
   *   if needed.  The NoteList.totalNotes field in the return value will
   *   indicate whether there are more values available after the returned set.
   *                                                      
   * @param fillScores
   *   Useful only for debugging and tuning, fill in the raw scores
   *   with each result.  This is expensive, so don't do it unless you are
   *   debugging or tuning.
   *
   * @param fillExplanation
   *   Useful only for debugging and tuning, fill in, for each result/hit,
   *   an explanation for how the result was scored.  This is expensive,
   *   so only set this to "true" when debugging or tuning. 
   *
   * @param fillQuery
   *   Useful only for debugging and tuning, indicates the server should
   *   fill in a string description of the native query issued against the
   *   native database of the server.
   */
  InternalRelatedNotesResult findRelatedNotesInternal(1: string authenticationToken,
                                                      2: InternalRelatedNotesQuery query,
                                                      4: i32 maxNotes,
                                                      5: bool fillScores,
                                                      6: bool fillExplanation,
                                                      7: bool fillQuery)
    throws (1: Errors.EDAMUserException userException,
            2: Errors.EDAMNotFoundException notFoundException,
            3: Errors.EDAMSystemException systemException)

 }