/*
 * Copyright 2011 Evernote Corporation, all rights reserved.
 */

/*
 * This file contains EDAM protocol definitions for internal, sharing-related
 * operations. These interfaces are not exposed in the official API published
 * for third party consumption.
 */

include "Errors.thrift"
include "Types.thrift"

namespace java com.evernote.edam.sharing
namespace csharp Evernote.EDAM.Sharing
namespace py evernote.edam.sharing
namespace cpp evernote.edam
namespace rb Evernote.EDAM.Sharing
namespace perl EDAMSharing
namespace php EDAM.Sharing

// ============================= Field Limits ==================================

// Access Token max length: 2048
// scope max length: 4096

/** Maximum OAuth Access Token Length */
const i32 EDAM_OAUTH_ACCESS_TOKEN_LEN_MAX = 2048;

/** Minimum OAuth Access Token length */
const i32 EDAM_OAUTH_ACCESS_TOKEN_LEN_MIN = 1;

/** Maximum OAuth Scope Length */
const i32 EDAM_OAUTH_SCOPE_LEN_MAX = 4096;

/** Minimum OAuth Scope Length */
const i32 EDAM_OAUTH_SCOPE_LEN_MIN = 0;

/** Maximum OAuth Service ID Length */
const i32 EDAM_OAUTH_SERVICE_ID_LEN_MAX = 8;

/** Minimum OAuth Service ID Length */
const i32 EDAM_OAUTH_SERVICE_ID_LEN_MIN = 1;

/** OAuth Version 1 */
const i16 EDAM_OAUTH_VERSION_1 = 1;

/** OAuth Version 2 */
const i16 EDAM_OAUTH_VERSION_2 = 2;

/** Acceptable OAuth Versions */
const set<i16> EDAM_OAUTH_VERSIONS = [
 EDAM_OAUTH_VERSION_1,
 EDAM_OAUTH_VERSION_2
];

// ============================== Structures ===================================

/**
 * An OAuth credential used to authenticate with a third party API.
 *
 * <dl>
 *   <dt>serviceId</dt>
 *     <dd>
 *     Identifies the service to which this credential applies.
 *     </dd>
 *
 *   <dt>oauthVer<dt>
 *     <dd>
 *     The OAuth version (1 or 2) to which this credential applies.
 *     </dd>
 *
 *   <dt>accessToken</dt>
 *     <dd>
 *     The access token used to authenticate with the API.
 *     </dd>
 *
 *   <dt>scope</dt>
 *     <dd>
 *     The scope of the access token.
 *     </dd>
 *
 *   <dt>created</dt>
 *     <dd>
 *     The creation time for this credential.
 *     </dd>
 *
 *   <dt>updated</dt>
 *     <dd>
 *     The last-updated time for this credential.
 *     </dd>
 */
struct OAuthCredential {
  2: optional string serviceId,
  3: optional i16 oauthVer,
  4: optional string accessToken,
  5: optional string scope,
  6: optional Types.Timestamp created,
  7: optional Types.Timestamp updated
}

// ============================== Service ===================================

/**
 * Service: Sharing
 *
 * Authenticated, sharing-related operations.
 */
service Sharing {
  // ====================== OAuth Credentials =================================

  /**
   * Returns true iff the user associated with the provided authentication token
   * has permission to send email to 'recipientCount' email recipients. If true,
   * his number will also be added to the count of emails sent by that user
   * today.
   */
  bool confirmSendEmail(1: string authenticationToken,
                           2: i32 recipientCount)
      throws (1: Errors.EDAMNotFoundException notFoundException,
              2: Errors.EDAMSystemException systemException,
              3: Errors.EDAMUserException userException),

  /**
   * Create OAuth Credential for User with Remote Service.
   */
  OAuthCredential createOAuthCredential(1: string authenticationToken,
                                        2: OAuthCredential credential)
   throws (1: Errors.EDAMNotFoundException notFoundException,
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException),

  /**
   * Remove OAuth Credential for User with Remote Service.
   */
  void deleteOAuthCredential(1: string authenticationToken,
                             2: string serviceId)
   throws (1: Errors.EDAMNotFoundException notFoundException
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException),

  /**
   * Retrieve OAuth Credential metadata for User with Remote Service.
   */
  OAuthCredential getOAuthCredential(1: string authenticationToken,
                                     2: string serviceId)
   throws (1: Errors.EDAMNotFoundException notFoundException,
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException),

  /**
   * Retrieve OAuth Credentials metadata for User.
   */
  list<OAuthCredential> getOAuthCredentials(1: string authenticationToken)
   throws (1: Errors.EDAMNotFoundException notFoundException,
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException),

  /**
   * Send an email referral message to each of the given email addresses.
   */
   void sendReferral(1: string authenticationToken,
                     2: list<string> addresses,
                     3: string message,
                     4: bool ccSender)
   throws (1: Errors.EDAMNotFoundException notFoundException,
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException),

  /**
   * Update existing OAuth Credential for User with Remote Service.
   */
  OAuthCredential updateOAuthCredential(1: string authenticationToken,
                                        2: OAuthCredential credential)
   throws (1: Errors.EDAMNotFoundException notFoundException,
           2: Errors.EDAMSystemException systemException,
           3: Errors.EDAMUserException userException)
}